#!/usr/bin/env python

import pybullet as p
import time
from pathlib import Path
from math import pi

urdf_folder = Path(__file__).parent / "../../urdf"

class CollisionChecker:
    """
    wrapper for pyBullet simulation
    """

    def __init__(self,
            numRobots = 2,
            robotPos = [
                ([0.5, 0, 0.001], [0,0,pi]),
                ([-0.5, 0, 0.001], [0,0,0])
                ],
            gui = True
            ):
        """
        RobotPos is a list of tuples, each tuple is:
        (RobotPosition, RobotOrientation)
        e.g. ([0,0,0], [0,0,pi])
        """

        assert (len(robotPos)==numRobots), "Incorrect number of robot poses"

        # start pybullet
        if gui:
            self.client = p.connect(p.GUI)
        else:
            self.client = p.connect(p.DIRECT)

        # Load Table
        self.table = p.loadURDF(str(urdf_folder/"table.urdf"),
                [0,0,0],
                p.getQuaternionFromEuler([0,0,0])
                )

        # Load Arms
        self.numRobots = numRobots
        self.robots = []
        for i in range(numRobots):
            self.robots.append(p.loadURDF(str(urdf_folder/"ur5e.urdf"),
                    robotPos[i][0],
                    p.getQuaternionFromEuler(robotPos[i][1])
                    ))


    def setJoints(self, r, joints):
        """
        set the rth robot's joint angles
        """
        for j in range(len(joints)):
            p.resetJointState(self.robots[r], j+1, joints[j])
        return


    def check(self, robots_joints): 
        """
        given joint angles of all robots, is there a collision?
        arguement is a list (robots) of lists (joint angles)
        """
        for r in range(self.numRobots):
            self.setJoints(r, robots_joints[r])

        p.stepSimulation()
        return len(p.getContactPoints()) - 2 # function returns 2 if no collisions

