import gcode2contour as gc
import numpy as np
from bisect import bisect_left
from trajectory_msgs.msg import JointTrajectory, JointTrajectoryPoint


def _emptyTraj(joints):
    """
    Creates an JointTrajectory with one point
    """
    traj = JointTrajectory()
    p = JointTrajectoryPoint()
    p.positions = joints
    traj.points.append(p)
    return traj


class Planner:
    """
    Holds the execution plan for n arms
    """

    def __init__(self,
            contour_tracker,
            collision_checker,
            n_arms,
            start_joints
            ):

        self.contour_tracker = contour_tracker
        self.cc = collision_checker
        self.n = n_arms
        self.trajs = {i:[_emptyTraj(start_joints[i])] for i in range(n_arms)}
        self.start_joints = start_joints

        # time at the end of each trajectory
        self.cumul_time = {i:[0] for i in range(n_arms)}


    def jointsAtTime(self, time, arm):
        """
        state of joints for an arm given time from start
        """

        # if no plan yet, still at start_joints
        if len(self.trajs[arm])==1 or time==0:
            return self.start_joints[arm]

        nth_traj = bisect_left(self.cumul_time[arm], time)
        if nth_traj == len(self.cumul_time[arm]):
            # return the final state of arm
            return self.trajs[arm][-1].points[-1].positions

        # else search for time in nth_traj
        tp = self.trajs[arm][nth_traj].points
        c_t = self.cumul_time[arm][nth_traj-1]
        wp_i = next(i for i,v in enumerate(tp)
                if v.time_from_start.to_sec() + c_t >= time)

        # linear interpolate between the waypoints
        j1 = np.array(tp[wp_i-1].positions)
        j2 = np.array(tp[wp_i].positions)
        t1 = tp[wp_i-1].time_from_start.to_sec() + c_t
        t2 = tp[wp_i].time_from_start.to_sec() + c_t

        return (time-t1)/(t2-t1)*(j2-j1) + j1


    def appendTrajectory(self, trajectories, arm, cc_res=0.5):
        """
        verify that trajectories can be added to an arm's plan
          and add them if no collisions.

        cc_res is time interval between collision checks in seconds
        """
        assert all([isinstance(t, JointTrajectory) for t in trajectories]), \
                "Input is not a list of JointTrajectories"

        # TODO check that trajectories start where last one ended

        # add trajectories to plan temporarily
        self.trajs[arm] = self.trajs[arm] + trajectories
        for traj in trajectories:
            self.cumul_time[arm].append(self.cumul_time[arm][-1] +
                    traj.points[-1].time_from_start.to_sec())

        # collision check along newly added
        # TODO move to binary search style traversal (faster collision finding)
        for t in np.arange(
                self.cumul_time[arm][-1-len(trajectories)],
                self.cumul_time[arm][-1],
                cc_res):
            joints_at_t = [self.jointsAtTime(t, arm) for arm in range(self.n)]
            if self.cc.check(joints_at_t) > 0:
                # remove trajectories from plan
                del self.trajs[arm][-len(trajectories):]
                del self.cumul_time[arm][-len(trajectories):]

                return 1 # 1 to indicate failed

        # If reached this point, no collision, trajs remain in plan
        return 0 # success

